# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.ext.commands.context import (
    MISSING,
    TYPE_CHECKING,
    Any,
    BotT,
    CogT,
    Context,
    Dict,
    Generic,
    List,
    Message,
    Optional,
    P,
    T,
    TypeVar,
    Union,
    annotations,
    inspect,
    re,
    versacord,
)

__all__ = "Context"
