# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.types.interactions import (
    TYPE_CHECKING,
    ApplicationCommand,
    ApplicationCommandInteractionData,
    ApplicationCommandInteractionDataOption,
    ApplicationCommandInteractionDataResolved,
    ApplicationCommandOption,
    ApplicationCommandOptionChoice,
    ApplicationCommandOptionType,
    ApplicationCommandPermissions,
    ApplicationCommandPermissionType,
    ApplicationCommandResolvedPartialChannel,
    ApplicationCommandType,
    BaseGuildApplicationCommandPermissions,
    ChannelType,
    Component,
    ComponentInteractionData,
    ComponentType,
    Dict,
    EditApplicationCommand,
    Embed,
    GuildApplicationCommandPermissions,
    Interaction,
    InteractionApplicationCommandCallbackData,
    InteractionData,
    InteractionResponse,
    InteractionResponseType,
    InteractionType,
    List,
    Literal,
    Member,
    MessageInteraction,
    Optional,
    PartialGuildApplicationCommandPermissions,
    Role,
    Snowflake,
    TypedDict,
    Union,
    User,
    _ApplicationCommandInteractionDataOption,
    _ApplicationCommandInteractionDataOptional,
    _ApplicationCommandInteractionDataOptionBoolean,
    _ApplicationCommandInteractionDataOptionInteger,
    _ApplicationCommandInteractionDataOptionNumber,
    _ApplicationCommandInteractionDataOptionSnowflake,
    _ApplicationCommandInteractionDataOptionString,
    _ApplicationCommandInteractionDataOptionSubcommand,
    _ApplicationCommandOptional,
    _ApplicationCommandOptionOptional,
    _ComponentInteractionDataOptional,
    _EditApplicationCommandOptional,
    _InteractionOptional,
    _InteractionResponseOptional,
    annotations,
)
