# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.types.components import (
    ActionRow,
    ButtonComponent,
    ButtonStyle,
    Component,
    ComponentType,
    List,
    Literal,
    PartialEmoji,
    SelectMenu,
    SelectOption,
    TypedDict,
    Union,
    _ButtonComponentOptional,
    _SelectMenuOptional,
    _SelectOptionsOptional,
    annotations,
)
