# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.ui.button import (
    TYPE_CHECKING,
    B,
    Button,
    ButtonComponent,
    ButtonStyle,
    Callable,
    ComponentType,
    Item,
    ItemCallbackType,
    Optional,
    PartialEmoji,
    Tuple,
    Type,
    TypeVar,
    Union,
    V,
    _EmojiTag,
    annotations,
    button,
    inspect,
    os,
)

__all__ = ("Button", "button")
