# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.file import TYPE_CHECKING, File, Optional, Union, annotations, io, os

__all__ = "File"
