# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.permissions import (
    PO,
    TYPE_CHECKING,
    Any,
    BaseFlags,
    Callable,
    ClassVar,
    Dict,
    Iterator,
    Optional,
    P,
    PermissionOverwrite,
    Permissions,
    Set,
    Tuple,
    Type,
    TypeVar,
    _augment_from_permissions,
    alias_flag_value,
    annotations,
    fill_with_flags,
    flag_value,
    make_permission_alias,
    permission_alias,
)

__all__ = ("Permissions", "PermissionOverwrite")
