# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.abc import (
    GCH,
    MISSING,
    TYPE_CHECKING,
    AllowedMentions,
    Any,
    Callable,
    ChannelType,
    ClientException,
    Connectable,
    Dict,
    File,
    GuildChannel,
    GuildSticker,
    HistoryIterator,
    InvalidArgument,
    Invite,
    List,
    Messageable,
    Optional,
    PermissionOverwrite,
    Permissions,
    PrivateChannel,
    Protocol,
    Role,
    Sequence,
    Snowflake,
    StickerItem,
    T,
    Tuple,
    TypeVar,
    Typing,
    Union,
    User,
    VoiceClient,
    VoiceProtocol,
    _Overwrites,
    _Undefined,
    _undefined,
    annotations,
    asyncio,
    copy,
    overload,
    runtime_checkable,
    utils,
)

__all__ = ("Snowflake", "User", "PrivateChannel", "GuildChannel", "Messageable", "Connectable")
