# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.application_command import (
    MISSING,
    TYPE_CHECKING,
    Any,
    ApplicationCheckFailure,
    ApplicationCommand,
    ApplicationCommandOptionType,
    ApplicationCommandType,
    ApplicationError,
    ApplicationInvokeError,
    ApplicationSubcommand,
    Attachment,
    Callable,
    ChannelType,
    ClientCog,
    CommandOption,
    Dict,
    FuncT,
    Guild,
    GuildChannel,
    Interaction,
    InvalidCommandType,
    Iterable,
    List,
    Member,
    Message,
    Optional,
    Parameter,
    Role,
    Set,
    SlashOption,
    T,
    Tuple,
    TypeVar,
    Union,
    User,
    _cog_special_method,
    annotations,
    asyncio,
    check_dictionary_values,
    deep_dictionary_check,
    find,
    maybe_coroutine,
    message_command,
    signature,
    slash_command,
    typing,
    user_command,
)

__all__ = (
    "ApplicationCommand",
    "ApplicationSubcommand",
    "ClientCog",
    "CommandOption",
    "message_command",
    "SlashOption",
    "slash_command",
    "user_command",
)
