# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.embeds import (
    TYPE_CHECKING,
    Any,
    Colour,
    Dict,
    E,
    Embed,
    EmbedProxy,
    EmptyEmbed,
    Final,
    List,
    Mapping,
    Protocol,
    Type,
    TypeVar,
    Union,
    _EmptyEmbed,
    annotations,
    datetime,
    utils,
)

__all__ = "Embed"
