# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.ui.item import (
    TYPE_CHECKING,
    Any,
    Callable,
    Coroutine,
    Dict,
    Generic,
    I,
    Interaction,
    Item,
    ItemCallbackType,
    Optional,
    Tuple,
    Type,
    TypeVar,
    V,
    annotations,
)

__all__ = "Item"
