# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.http import (
    MISSING,
    TYPE_CHECKING,
    Any,
    ClassVar,
    Coroutine,
    Dict,
    DiscordClientWebSocketResponse,
    DiscordServerError,
    Forbidden,
    GatewayNotFound,
    HTTPClient,
    HTTPException,
    InvalidArgument,
    Iterable,
    List,
    LoginFailure,
    MaybeUnlock,
    NotFound,
    Optional,
    Route,
    Sequence,
    Tuple,
    Type,
    TypeVar,
    Union,
    _log,
    _uriquote,
    aiohttp,
    annotations,
    asyncio,
    json,
    json_or_text,
    logging,
    sys,
    utils,
    weakref,
)
