# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.webhook.__init__ import (
    PartialWebhookChannel,
    PartialWebhookGuild,
    SyncWebhook,
    SyncWebhookMessage,
    Webhook,
    WebhookMessage,
)
