# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.colour import (
    CT,
    Any,
    Color,
    Colour,
    Optional,
    Tuple,
    Type,
    TypeVar,
    Union,
    colorsys,
    random,
)

__all__ = ("Colour", "Color")
