# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.asset import (
    MISSING,
    TYPE_CHECKING,
    VALID_ASSET_FORMATS,
    VALID_STATIC_FORMATS,
    Any,
    Asset,
    AssetMixin,
    DiscordException,
    InvalidArgument,
    Literal,
    Optional,
    Tuple,
    Union,
    annotations,
    io,
    os,
    utils,
    yarl,
)

__all__ = "Asset"
