# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.ext.commands._types import (
    TYPE_CHECKING,
    Any,
    Callable,
    Check,
    Coro,
    CoroFunc,
    Coroutine,
    Error,
    Hook,
    MaybeCoro,
    T,
    TypeVar,
    Union,
    _BaseCommand,
)
