# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.gateway import (
    BaseActivity,
    ConnectionClosed,
    DiscordClientWebSocketResponse,
    DiscordVoiceWebSocket,
    DiscordWebSocket,
    EventListener,
    GatewayRatelimiter,
    InvalidArgument,
    KeepAliveHandler,
    ReconnectWebSocket,
    SpeakingState,
    VoiceKeepAliveHandler,
    WebSocketClosure,
    _log,
    aiohttp,
    asyncio,
    concurrent,
    deque,
    logging,
    namedtuple,
    struct,
    sys,
    threading,
    time,
    traceback,
    utils,
    zlib,
)

__all__ = (
    "DiscordWebSocket",
    "KeepAliveHandler",
    "VoiceKeepAliveHandler",
    "DiscordVoiceWebSocket",
    "ReconnectWebSocket",
)
