# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.types.raw_models import (
    BulkMessageDeleteEvent,
    IntegrationDeleteEvent,
    List,
    Member,
    MessageDeleteEvent,
    MessageUpdateEvent,
    PartialEmoji,
    ReactionActionEvent,
    ReactionClearEmojiEvent,
    ReactionClearEvent,
    Snowflake,
    TypedDict,
    TypingEvent,
    _IntegrationDeleteEventOptional,
    _MessageEventOptional,
    _ReactionActionEventOptional,
    _ReactionClearEmojiEventOptional,
    _ReactionClearEventOptional,
    _TypingEventOptional,
)
