# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.shard import (
    TYPE_CHECKING,
    Any,
    AutoShardedClient,
    AutoShardedConnectionState,
    Callable,
    Client,
    ClientException,
    ConnectionClosed,
    Dict,
    DiscordVoiceWebSocket,
    DiscordWebSocket,
    EventItem,
    EventType,
    ExponentialBackoff,
    GatewayNotFound,
    HTTPException,
    KeepAliveHandler,
    List,
    Optional,
    PrivilegedIntentsRequired,
    ReconnectWebSocket,
    Shard,
    ShardInfo,
    Status,
    Tuple,
    Type,
    TypeVar,
    VoiceKeepAliveHandler,
    _log,
    aiohttp,
    annotations,
    asyncio,
    logging,
)

__all__ = ("AutoShardedClient", "ShardInfo")
