# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.activity import (
    TYPE_CHECKING,
    Activity,
    ActivityType,
    ActivityTypes,
    Any,
    Asset,
    BaseActivity,
    Colour,
    CustomActivity,
    Dict,
    Game,
    List,
    Optional,
    PartialEmoji,
    Spotify,
    Streaming,
    Union,
    annotations,
    create_activity,
    datetime,
    get_as_snowflake,
    overload,
    try_enum,
)

__all__ = ("BaseActivity", "Activity", "Streaming", "Game", "Spotify", "CustomActivity")
