# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.health_check import (
    DistributionNotFound,
    DistributionWarning,
    get_distribution,
    incompatible_libraries,
    library,
    warn,
)

__all__ = "incompatible_libraries"
