# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.widget import (
    TYPE_CHECKING,
    Activity,
    Any,
    BaseActivity,
    BaseUser,
    Invite,
    List,
    Optional,
    Spotify,
    Status,
    Union,
    Widget,
    WidgetChannel,
    WidgetMember,
    annotations,
    create_activity,
    get_as_snowflake,
    resolve_invite,
    snowflake_time,
    try_enum,
)

__all__ = ("WidgetChannel", "WidgetMember", "Widget")
