# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.webhook.sync import (
    MISSING,
    TYPE_CHECKING,
    Any,
    BaseWebhook,
    DeferredLock,
    Dict,
    DiscordServerError,
    Forbidden,
    HTTPException,
    InvalidArgument,
    List,
    Literal,
    Message,
    NotFound,
    Optional,
    PartialMessageable,
    Route,
    SyncWebhook,
    SyncWebhookMessage,
    Tuple,
    Type,
    TypeVar,
    Union,
    WebhookAdapter,
    _context,
    _get_webhook_adapter,
    _log,
    _WebhookContext,
    _WebhookState,
    annotations,
    handle_message_parameters,
    json,
    logging,
    overload,
    re,
    threading,
    time,
    urlquote,
    utils,
)

__all__ = ("SyncWebhook", "SyncWebhookMessage")
