# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.player import (
    AT,
    CREATE_NO_WINDOW,
    FT,
    IO,
    MISSING,
    TYPE_CHECKING,
    Any,
    AudioPlayer,
    AudioSource,
    Callable,
    ClientException,
    FFmpegAudio,
    FFmpegOpusAudio,
    FFmpegPCMAudio,
    Generic,
    OggStream,
    Optional,
    OpusEncoder,
    PCMAudio,
    PCMVolumeTransformer,
    Tuple,
    Type,
    TypeVar,
    Union,
    _log,
    annotations,
    asyncio,
    audioop,
    io,
    json,
    logging,
    re,
    shlex,
    subprocess,
    sys,
    threading,
    time,
    traceback,
)

__all__ = (
    "AudioSource",
    "PCMAudio",
    "FFmpegAudio",
    "FFmpegPCMAudio",
    "FFmpegOpusAudio",
    "PCMVolumeTransformer",
)
