# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.message import (
    MISSING,
    TYPE_CHECKING,
    Any,
    Attachment,
    Callable,
    ChannelType,
    ClassVar,
    DeletedReferencedMessage,
    Dict,
    Embed,
    Emoji,
    File,
    Guild,
    Hashable,
    HTTPException,
    InvalidArgument,
    List,
    Member,
    Message,
    MessageFlags,
    MessageReference,
    MessageType,
    Optional,
    PartialEmoji,
    PartialMessage,
    PathLike,
    Reaction,
    StickerItem,
    Thread,
    Tuple,
    Type,
    TypeVar,
    Union,
    _component_factory,
    annotations,
    asyncio,
    convert_emoji_reaction,
    datetime,
    escape_mentions,
    flatten_handlers,
    io,
    overload,
    re,
    try_enum,
    utils,
)

__all__ = (
    "Attachment",
    "Message",
    "PartialMessage",
    "MessageReference",
    "DeletedReferencedMessage",
)
