# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.guild import (
    MISSING,
    TYPE_CHECKING,
    Any,
    Asset,
    AuditLogAction,
    AuditLogIterator,
    BanEntry,
    CategoryChannel,
    ChannelType,
    ClassVar,
    ClientException,
    Colour,
    ContentFilter,
    Dict,
    DMChannel,
    Emoji,
    File,
    GroupChannel,
    Guild,
    GuildSticker,
    Hashable,
    Integration,
    InvalidArgument,
    InvalidData,
    Invite,
    List,
    Literal,
    Member,
    MemberIterator,
    NamedTuple,
    NotificationLevel,
    NSFWLevel,
    Optional,
    PartialMessageable,
    PermissionOverwrite,
    Role,
    Sequence,
    Set,
    StageChannel,
    StageInstance,
    SystemChannelFlags,
    TextChannel,
    Thread,
    ThreadMember,
    Tuple,
    Union,
    User,
    VerificationLevel,
    VideoQualityMode,
    VoiceChannel,
    VoiceRegion,
    VoiceState,
    Widget,
    _guild_channel_factory,
    _GuildLimit,
    _integration_factory,
    _threaded_guild_channel_factory,
    abc,
    annotations,
    copy,
    overload,
    try_enum,
    unicodedata,
    utils,
)

__all__ = "Guild"
