# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.role import (
    MISSING,
    TYPE_CHECKING,
    Any,
    Colour,
    Dict,
    Hashable,
    InvalidArgument,
    List,
    Optional,
    Permissions,
    R,
    Role,
    RoleTags,
    TypeVar,
    Union,
    annotations,
    get_as_snowflake,
    overload,
    snowflake_time,
)

__all__ = ("RoleTags", "Role")
