# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.ext.commands.core import (
    MISSING,
    TYPE_CHECKING,
    Any,
    ArgumentParsingError,
    BadArgument,
    BadBoolArgument,
    BadColorArgument,
    BadColourArgument,
    BadFlagArgument,
    BadInviteArgument,
    BadLiteralArgument,
    BadUnionArgument,
    BotMissingAnyRole,
    BotMissingPermissions,
    BotMissingRole,
    BucketType,
    Callable,
    ChannelNotFound,
    ChannelNotReadable,
    CheckAnyFailure,
    CheckFailure,
    Cog,
    CogT,
    Command,
    CommandError,
    CommandInvokeError,
    CommandNotFound,
    CommandOnCooldown,
    CommandRegistrationError,
    CommandT,
    Context,
    ContextT,
    ConversionError,
    Cooldown,
    CooldownMapping,
    Dict,
    DisabledCommand,
    DynamicCooldownMapping,
    EmojiNotFound,
    ErrorT,
    ExpectedClosingQuoteError,
    ExtensionAlreadyLoaded,
    ExtensionError,
    ExtensionFailed,
    ExtensionNotFound,
    ExtensionNotLoaded,
    FlagError,
    Generator,
    Generic,
    Greedy,
    Group,
    GroupMixin,
    GroupT,
    GuildNotFound,
    GuildStickerNotFound,
    HookT,
    InvalidEndOfQuotedStringError,
    List,
    Literal,
    MaxConcurrency,
    MaxConcurrencyReached,
    MemberNotFound,
    MessageNotFound,
    MissingAnyRole,
    MissingFlagArgument,
    MissingPermissions,
    MissingRequiredArgument,
    MissingRequiredFlag,
    MissingRole,
    NoEntryPointError,
    NoPrivateMessage,
    NotOwner,
    NSFWChannelRequired,
    ObjectNotFound,
    Optional,
    P,
    PartialEmojiConversionFailure,
    PrivateMessageOnly,
    RoleNotFound,
    Set,
    T,
    ThreadNotFound,
    TooManyArguments,
    TooManyFlags,
    Tuple,
    Type,
    TypeVar,
    UnexpectedQuoteError,
    Union,
    UserInputError,
    UserNotFound,
    _BaseCommand,
    _CaseInsensitiveDict,
    after_invoke,
    annotations,
    asyncio,
    before_invoke,
    bot_has_any_role,
    bot_has_guild_permissions,
    bot_has_permissions,
    bot_has_role,
    check,
    check_any,
    command,
    cooldown,
    datetime,
    dm_only,
    dynamic_cooldown,
    functools,
    get_converter,
    get_signature_parameters,
    group,
    guild_only,
    has_any_role,
    has_guild_permissions,
    has_permissions,
    has_role,
    hooked_wrapped_callback,
    inspect,
    is_nsfw,
    is_owner,
    max_concurrency,
    overload,
    run_converters,
    unwrap_function,
    versacord,
    wrap_callback,
)

__all__ = (
    "Command",
    "Group",
    "GroupMixin",
    "command",
    "group",
    "has_role",
    "has_permissions",
    "has_any_role",
    "check",
    "check_any",
    "before_invoke",
    "after_invoke",
    "bot_has_role",
    "bot_has_permissions",
    "bot_has_any_role",
    "cooldown",
    "dynamic_cooldown",
    "max_concurrency",
    "dm_only",
    "guild_only",
    "is_owner",
    "is_nsfw",
    "has_guild_permissions",
    "bot_has_guild_permissions",
)
