# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.member import (
    MISSING,
    TYPE_CHECKING,
    ActivityTypes,
    Any,
    Asset,
    BaseUser,
    Colour,
    Dict,
    List,
    Literal,
    M,
    Member,
    Object,
    Optional,
    Permissions,
    Status,
    Tuple,
    Type,
    TypeVar,
    Union,
    User,
    VoiceState,
    _UserTag,
    abc,
    annotations,
    attrgetter,
    create_activity,
    datetime,
    flatten_user,
    inspect,
    itertools,
    overload,
    sys,
    try_enum,
    utils,
)

__all__ = ("VoiceState", "Member")
