# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.errors import (
    TYPE_CHECKING,
    Any,
    ClientException,
    ConnectionClosed,
    Dict,
    DiscordException,
    DiscordServerError,
    Forbidden,
    GatewayNotFound,
    HTTPException,
    InteractionResponded,
    InvalidArgument,
    InvalidData,
    List,
    LoginFailure,
    NoMoreItems,
    NotFound,
    Optional,
    PrivilegedIntentsRequired,
    Tuple,
    Union,
    _flatten_error_dict,
    annotations,
)

__all__ = (
    "DiscordException",
    "ClientException",
    "NoMoreItems",
    "GatewayNotFound",
    "HTTPException",
    "Forbidden",
    "NotFound",
    "DiscordServerError",
    "InvalidData",
    "InvalidArgument",
    "LoginFailure",
    "ConnectionClosed",
    "PrivilegedIntentsRequired",
    "InteractionResponded",
)
