# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.ext.commands.flags import (
    MISSING,
    TYPE_CHECKING,
    Any,
    BadFlagArgument,
    CommandError,
    Dict,
    F,
    Flag,
    FlagConverter,
    FlagsMeta,
    Iterator,
    List,
    Literal,
    MissingFlagArgument,
    MissingRequiredFlag,
    Optional,
    Pattern,
    Set,
    StringView,
    TooManyFlags,
    Tuple,
    Type,
    TypeVar,
    Union,
    annotations,
    convert_flag,
    dataclass,
    field,
    flag,
    get_flags,
    inspect,
    maybe_coroutine,
    re,
    resolve_annotation,
    run_converters,
    sys,
    tuple_convert_all,
    tuple_convert_flag,
    validate_flag_name,
)

__all__ = ("Flag", "flag", "FlagConverter")
