# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.types.invite import (
    GatewayInvite,
    GatewayInviteCreate,
    GatewayInviteDelete,
    IncompleteInvite,
    Invite,
    InviteGuild,
    InviteTargetType,
    InviteWithCounts,
    Literal,
    Optional,
    PartialAppInfo,
    PartialChannel,
    PartialUser,
    Snowflake,
    TypedDict,
    Union,
    VanityInvite,
    _GatewayInviteCreateOptional,
    _GatewayInviteDeleteOptional,
    _GuildPreviewUnique,
    _InviteMetadata,
    _InviteOptional,
    annotations,
)
