# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.oggparse import (
    IO,
    TYPE_CHECKING,
    ClassVar,
    DiscordException,
    Generator,
    OggError,
    OggPage,
    OggStream,
    Optional,
    Tuple,
    annotations,
    struct,
)

__all__ = ("OggError", "OggPage", "OggStream")
