# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.types.channel import (
    CategoryChannel,
    Channel,
    ChannelType,
    DMChannel,
    GroupDMChannel,
    GuildChannel,
    List,
    Literal,
    NewsChannel,
    Optional,
    OverwriteType,
    PartialChannel,
    PartialUser,
    PermissionOverwrite,
    PrivacyLevel,
    Snowflake,
    StageChannel,
    StageInstance,
    TextChannel,
    ThreadArchiveDuration,
    ThreadChannel,
    ThreadMember,
    ThreadMetadata,
    TypedDict,
    Union,
    VideoQualityMode,
    VoiceChannel,
    _BaseChannel,
    _BaseGuildChannel,
    _StageChannelOptional,
    _TextChannelOptional,
    _ThreadChannelOptional,
    _VoiceChannelOptional,
)
