# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.types.webhook import (
    FollowerWebhook,
    Literal,
    Optional,
    PartialChannel,
    PartialWebhook,
    Snowflake,
    SourceGuild,
    TypedDict,
    User,
    Webhook,
    WebhookType,
    _FollowerWebhookOptional,
    _FullWebhook,
    _WebhookOptional,
    annotations,
)
