# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.template import (
    MISSING,
    TYPE_CHECKING,
    Any,
    Guild,
    Optional,
    Template,
    VoiceRegion,
    _bytes_to_base64_data,
    _FriendlyHttpAttributeErrorHelper,
    _PartialTemplateState,
    annotations,
    get_as_snowflake,
    parse_time,
)

__all__ = "Template"
