# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.audit_logs import (
    TYPE_CHECKING,
    Any,
    Asset,
    AuditLogChanges,
    AuditLogDiff,
    AuditLogEntry,
    Callable,
    ClassVar,
    Colour,
    Dict,
    Generator,
    Hashable,
    Invite,
    List,
    Object,
    Optional,
    PermissionOverwrite,
    Permissions,
    T,
    Transformer,
    Tuple,
    Type,
    TypeVar,
    Union,
    _AuditLogProxyMemberDisconnect,
    _AuditLogProxyMemberMoveOrMessageDelete,
    _AuditLogProxyMemberPrune,
    _AuditLogProxyPinAction,
    _AuditLogProxyStageInstanceAction,
    _enum_transformer,
    _guild_hash_transformer,
    _transform_avatar,
    _transform_channel,
    _transform_color,
    _transform_guild_id,
    _transform_icon,
    _transform_member_id,
    _transform_overwrites,
    _transform_permissions,
    _transform_snowflake,
    _transform_type,
    annotations,
    enums,
    utils,
)

__all__ = ("AuditLogDiff", "AuditLogChanges", "AuditLogEntry")
