# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.state import (
    TYPE_CHECKING,
    AllowedMentions,
    Any,
    ApplicationFlags,
    AutoShardedConnectionState,
    BaseActivity,
    Callable,
    CategoryChannel,
    ChannelType,
    ChunkRequest,
    ClientUser,
    ConnectionState,
    Coroutine,
    Deque,
    Dict,
    DMChannel,
    Emoji,
    GroupChannel,
    Guild,
    GuildSticker,
    Intents,
    Interaction,
    Invite,
    List,
    Member,
    MemberCacheFlags,
    Message,
    Object,
    Optional,
    OrderedDict,
    PartialEmoji,
    PartialMessageable,
    RawBulkMessageDeleteEvent,
    RawIntegrationDeleteEvent,
    RawMessageDeleteEvent,
    RawMessageUpdateEvent,
    RawReactionActionEvent,
    RawReactionClearEmojiEvent,
    RawReactionClearEvent,
    RawTypingEvent,
    Role,
    Sequence,
    StageChannel,
    StageInstance,
    Status,
    TextChannel,
    Thread,
    ThreadMember,
    Tuple,
    TypeVar,
    Union,
    User,
    View,
    ViewStore,
    VoiceChannel,
    _channel_factory,
    _integration_factory,
    _log,
    annotations,
    asyncio,
    copy,
    datetime,
    deque,
    inspect,
    itertools,
    logging,
    logging_coroutine,
    os,
    try_enum,
    utils,
)
