# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.integrations import (
    MISSING,
    TYPE_CHECKING,
    Any,
    BotIntegration,
    Dict,
    ExpireBehaviour,
    Integration,
    IntegrationAccount,
    IntegrationApplication,
    InvalidArgument,
    Optional,
    StreamIntegration,
    Tuple,
    Type,
    User,
    _integration_factory,
    annotations,
    datetime,
    get_as_snowflake,
    overload,
    parse_time,
    try_enum,
)

__all__ = (
    "IntegrationAccount",
    "IntegrationApplication",
    "Integration",
    "StreamIntegration",
    "BotIntegration",
)
