# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.flags import (
    BF,
    FV,
    Any,
    ApplicationFlags,
    BaseFlags,
    Callable,
    ClassVar,
    Dict,
    Generic,
    Intents,
    Iterator,
    List,
    MemberCacheFlags,
    MessageFlags,
    Optional,
    PublicUserFlags,
    SystemChannelFlags,
    Tuple,
    Type,
    TypeVar,
    UserFlags,
    alias_flag_value,
    annotations,
    fill_with_flags,
    flag_value,
    overload,
)

__all__ = (
    "SystemChannelFlags",
    "MessageFlags",
    "PublicUserFlags",
    "Intents",
    "MemberCacheFlags",
    "ApplicationFlags",
)
