# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.types.threads import (
    List,
    Literal,
    Optional,
    Snowflake,
    Thread,
    ThreadArchiveDuration,
    ThreadMember,
    ThreadMetadata,
    ThreadPaginationPayload,
    ThreadType,
    TypedDict,
    _ThreadMetadataOptional,
    _ThreadOptional,
    annotations,
)
