# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.enums import (
    ActivityType,
    Any,
    AuditLogAction,
    AuditLogActionCategory,
    ButtonStyle,
    ChannelType,
    ComponentType,
    ContentFilter,
    DefaultAvatar,
    Dict,
    Enum,
    ExpireBehavior,
    ExpireBehaviour,
    IntEnum,
    InteractionResponseType,
    InteractionType,
    InviteTarget,
    MessageType,
    NotificationLevel,
    NSFWLevel,
    Optional,
    SpeakingState,
    StagePrivacyLevel,
    Status,
    StickerFormatType,
    StickerType,
    StrEnum,
    T,
    TeamMembershipState,
    Type,
    TypeVar,
    UnknownEnumValue,
    UserFlags,
    VerificationLevel,
    VideoQualityMode,
    VoiceRegion,
    WebhookType,
    try_enum,
)

__all__ = (
    "Enum",
    "IntEnum",
    "StrEnum",
    "UnknownEnumValue",
    "ChannelType",
    "MessageType",
    "VoiceRegion",
    "SpeakingState",
    "VerificationLevel",
    "ContentFilter",
    "Status",
    "DefaultAvatar",
    "AuditLogAction",
    "AuditLogActionCategory",
    "UserFlags",
    "ActivityType",
    "NotificationLevel",
    "TeamMembershipState",
    "WebhookType",
    "ExpireBehaviour",
    "ExpireBehavior",
    "StickerType",
    "StickerFormatType",
    "InviteTarget",
    "VideoQualityMode",
    "ComponentType",
    "ButtonStyle",
    "StagePrivacyLevel",
    "InteractionType",
    "InteractionResponseType",
    "NSFWLevel",
)
