"""
versacord.webhook
~~~~~~~~~~~~~~~~

Webhook support

:copyright: (c) 2015-2021 Rapptz
:copyright: (c) 2022-present versacord Developers
:license: MIT, see LICENSE for more details.
"""

from .async_ import *
from .sync import *
