# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.opus import (
    APPLICATION_AUDIO,
    APPLICATION_LOWDELAY,
    APPLICATION_VOIP,
    BAD_ARG,
    CTL_LAST_PACKET_DURATION,
    CTL_SET_BANDWIDTH,
    CTL_SET_BITRATE,
    CTL_SET_FEC,
    CTL_SET_GAIN,
    CTL_SET_PLP,
    CTL_SET_SIGNAL,
    OK,
    TYPE_CHECKING,
    Any,
    BandCtl,
    Callable,
    Decoder,
    DecoderStruct,
    DecoderStructPtr,
    DiscordException,
    Encoder,
    EncoderStruct,
    EncoderStructPtr,
    InvalidArgument,
    List,
    Literal,
    Optional,
    OpusError,
    OpusNotLoaded,
    SignalCtl,
    Tuple,
    TypedDict,
    TypeVar,
    _err_lt,
    _err_ne,
    _lib,
    _load_default,
    _log,
    _OpusStruct,
    annotations,
    array,
    band_ctl,
    c_float_ptr,
    c_int16_ptr,
    c_int_ptr,
    ctypes,
    exported_functions,
    is_loaded,
    libopus_loader,
    load_opus,
    logging,
    math,
    os,
    overload,
    signal_ctl,
    struct,
    sys,
)

__all__ = ("Encoder", "OpusError", "OpusNotLoaded")
