# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.types.team import (
    List,
    Optional,
    PartialUser,
    Snowflake,
    Team,
    TeamMember,
    TypedDict,
    annotations,
)
