# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.ext.commands.cooldowns import (
    MC,
    TYPE_CHECKING,
    Any,
    BucketType,
    C,
    Callable,
    Cooldown,
    CooldownMapping,
    Deque,
    Dict,
    DynamicCooldownMapping,
    Enum,
    MaxConcurrency,
    MaxConcurrencyReached,
    Optional,
    PrivateChannel,
    Type,
    TypeVar,
    _Semaphore,
    annotations,
    asyncio,
    deque,
    time,
)

__all__ = ("BucketType", "Cooldown", "CooldownMapping", "DynamicCooldownMapping", "MaxConcurrency")
