# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.ext.commands.view import (
    ExpectedClosingQuoteError,
    InvalidEndOfQuotedStringError,
    StringView,
    UnexpectedQuoteError,
    _all_quotes,
    _quotes,
)
