# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.stage_instance import (
    MISSING,
    TYPE_CHECKING,
    Hashable,
    InvalidArgument,
    Optional,
    StageInstance,
    StagePrivacyLevel,
    annotations,
    cached_slot_property,
    try_enum,
)

__all__ = "StageInstance"
