# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.interactions import (
    MISSING,
    TYPE_CHECKING,
    Any,
    Attachment,
    ChannelType,
    ClientException,
    Dict,
    HTTPException,
    Interaction,
    InteractionMessage,
    InteractionResponded,
    InteractionResponse,
    InteractionResponseType,
    InteractionType,
    List,
    Member,
    Message,
    Object,
    Optional,
    PartialMessageable,
    Permissions,
    Tuple,
    Union,
    User,
    Webhook,
    _InteractionMessageState,
    annotations,
    async_context,
    asyncio,
    handle_message_parameters,
    try_enum,
    utils,
)

__all__ = ("Interaction", "InteractionMessage", "InteractionResponse")
