# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.invite import (
    TYPE_CHECKING,
    Asset,
    ChannelType,
    Hashable,
    I,
    Invite,
    InviteTarget,
    List,
    Object,
    Optional,
    PartialAppInfo,
    PartialInviteChannel,
    PartialInviteGuild,
    Type,
    TypeVar,
    Union,
    VerificationLevel,
    annotations,
    get_as_snowflake,
    parse_time,
    snowflake_time,
    try_enum,
)

__all__ = ("PartialInviteChannel", "PartialInviteGuild", "Invite")
