# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.backoff import (
    Callable,
    ExponentialBackoff,
    Generic,
    Literal,
    T,
    TypeVar,
    Union,
    annotations,
    overload,
    random,
    time,
)

__all__ = ("ExponentialBackoff",)
