# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.ext.commands.converter import (
    _ID_REGEX,
    CONVERTER_MAPPING,
    CT,
    TT,
    TYPE_CHECKING,
    Any,
    ArgumentParsingError,
    BadArgument,
    BadBoolArgument,
    BadColorArgument,
    BadColourArgument,
    BadFlagArgument,
    BadInviteArgument,
    BadLiteralArgument,
    BadUnionArgument,
    BotMissingAnyRole,
    BotMissingPermissions,
    BotMissingRole,
    CategoryChannelConverter,
    ChannelNotFound,
    ChannelNotReadable,
    CheckAnyFailure,
    CheckFailure,
    ColorConverter,
    ColourConverter,
    CommandError,
    CommandInvokeError,
    CommandNotFound,
    CommandOnCooldown,
    CommandRegistrationError,
    ConversionError,
    Converter,
    Dict,
    DisabledCommand,
    EmojiConverter,
    EmojiNotFound,
    ExpectedClosingQuoteError,
    ExtensionAlreadyLoaded,
    ExtensionError,
    ExtensionFailed,
    ExtensionNotFound,
    ExtensionNotLoaded,
    FlagError,
    GameConverter,
    Generic,
    Greedy,
    GuildChannelConverter,
    GuildConverter,
    GuildNotFound,
    GuildStickerConverter,
    GuildStickerNotFound,
    IDConverter,
    InvalidEndOfQuotedStringError,
    InviteConverter,
    Iterable,
    List,
    Literal,
    MaxConcurrencyReached,
    MemberConverter,
    MemberNotFound,
    MessageConverter,
    MessageNotFound,
    MissingAnyRole,
    MissingFlagArgument,
    MissingPermissions,
    MissingRequiredArgument,
    MissingRequiredFlag,
    MissingRole,
    NoEntryPointError,
    NoPrivateMessage,
    NotOwner,
    NSFWChannelRequired,
    ObjectConverter,
    ObjectNotFound,
    Optional,
    PartialEmojiConversionFailure,
    PartialEmojiConverter,
    PartialMessageConverter,
    PrivateMessageOnly,
    Protocol,
    RoleConverter,
    RoleNotFound,
    StageChannelConverter,
    T,
    T_co,
    TextChannelConverter,
    ThreadConverter,
    ThreadNotFound,
    TooManyArguments,
    TooManyFlags,
    Tuple,
    Type,
    TypeVar,
    UnexpectedQuoteError,
    Union,
    UserConverter,
    UserInputError,
    UserNotFound,
    VoiceChannelConverter,
    _actual_conversion,
    _convert_to_bool,
    _GenericAlias,
    _get_from_guilds,
    _utils_get,
    annotations,
    clean_content,
    get_converter,
    inspect,
    is_generic_type,
    re,
    run_converters,
    runtime_checkable,
    versacord,
)

__all__ = (
    "Converter",
    "ObjectConverter",
    "MemberConverter",
    "UserConverter",
    "MessageConverter",
    "PartialMessageConverter",
    "TextChannelConverter",
    "InviteConverter",
    "GuildConverter",
    "RoleConverter",
    "GameConverter",
    "ColourConverter",
    "ColorConverter",
    "VoiceChannelConverter",
    "StageChannelConverter",
    "EmojiConverter",
    "PartialEmojiConverter",
    "CategoryChannelConverter",
    "IDConverter",
    "ThreadConverter",
    "GuildChannelConverter",
    "GuildStickerConverter",
    "clean_content",
    "Greedy",
    "run_converters",
)
