# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.types.template import (
    CreateTemplate,
    Guild,
    Optional,
    Snowflake,
    Template,
    TypedDict,
    User,
    annotations,
)
