# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.user import (
    BU,
    MISSING,
    TYPE_CHECKING,
    Any,
    Asset,
    BaseUser,
    ClientUser,
    Colour,
    DefaultAvatar,
    Dict,
    List,
    Optional,
    PublicUserFlags,
    Type,
    TypeVar,
    User,
    _bytes_to_base64_data,
    _UserTag,
    abc,
    annotations,
    snowflake_time,
)

__all__ = ("User", "ClientUser")
