# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.sticker import (
    MISSING,
    TYPE_CHECKING,
    Asset,
    AssetMixin,
    GuildSticker,
    Hashable,
    InvalidData,
    List,
    Literal,
    Optional,
    StandardSticker,
    Sticker,
    StickerFormatType,
    StickerItem,
    StickerPack,
    StickerType,
    Tuple,
    Type,
    Union,
    _sticker_factory,
    _StickerTag,
    annotations,
    cached_slot_property,
    find,
    get,
    snowflake_time,
    try_enum,
    unicodedata,
)

__all__ = ("StickerPack", "StickerItem", "Sticker", "StandardSticker", "GuildSticker")
