# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.mentions import (
    TYPE_CHECKING,
    A,
    AllowedMentions,
    Any,
    List,
    Type,
    TypeVar,
    Union,
    _FakeBool,
    annotations,
    default,
)

__all__ = "AllowedMentions"
