# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.ui.select import (
    MISSING,
    TYPE_CHECKING,
    Callable,
    ComponentType,
    Emoji,
    Interaction,
    Item,
    ItemCallbackType,
    List,
    Optional,
    PartialEmoji,
    S,
    Select,
    SelectMenu,
    SelectOption,
    Tuple,
    Type,
    TypeVar,
    Union,
    V,
    annotations,
    inspect,
    os,
    select,
)

__all__ = ("Select", "select")
