# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.reaction import (
    TYPE_CHECKING,
    Any,
    Optional,
    Reaction,
    ReactionIterator,
    Union,
    annotations,
)

__all__ = "Reaction"
