# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.types.integration import (
    BaseIntegration,
    BotIntegration,
    Integration,
    IntegrationAccount,
    IntegrationApplication,
    IntegrationExpireBehavior,
    IntegrationType,
    Literal,
    Optional,
    PartialIntegration,
    Snowflake,
    StreamIntegration,
    TypedDict,
    Union,
    User,
    _IntegrationApplicationOptional,
    annotations,
)
