# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.types.message import (
    AllowedMentions,
    AllowedMentionType,
    Attachment,
    ChannelMention,
    ChannelType,
    Component,
    Embed,
    List,
    Literal,
    Member,
    Message,
    MessageActivity,
    MessageActivityType,
    MessageApplication,
    MessageInteraction,
    MessageReference,
    MessageType,
    Optional,
    PartialEmoji,
    Reaction,
    Snowflake,
    SnowflakeList,
    StickerItem,
    TypedDict,
    Union,
    User,
    UserWithMember,
    _AttachmentOptional,
    _MessageApplicationOptional,
    _MessageOptional,
    annotations,
)
