# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.voice_client import (
    MISSING,
    TYPE_CHECKING,
    Any,
    AudioPlayer,
    AudioSource,
    Callable,
    ClientException,
    ConnectionClosed,
    DiscordVoiceWebSocket,
    DiscordWebSocket,
    ExponentialBackoff,
    KeepAliveHandler,
    List,
    Optional,
    ReconnectWebSocket,
    Tuple,
    VoiceClient,
    VoiceKeepAliveHandler,
    VoiceProtocol,
    _log,
    annotations,
    asyncio,
    has_nacl,
    logging,
    opus,
    socket,
    struct,
    threading,
    utils,
)

__all__ = ("VoiceProtocol", "VoiceClient")

if has_nacl:
    from versacord.voice_client import nacl
