# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.welcome_screen import (
    TYPE_CHECKING,
    Dict,
    InvalidArgument,
    List,
    Optional,
    Union,
    WelcomeChannel,
    WelcomeScreen,
    _EmojiTag,
    annotations,
    get,
    get_as_snowflake,
    overload,
)

__all__ = ("WelcomeChannel", "WelcomeScreen")
