# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.ext.commands.bot import (
    CFT,
    CXT,
    MISSING,
    TYPE_CHECKING,
    Any,
    AutoShardedBot,
    Bot,
    BotBase,
    Callable,
    Cog,
    Context,
    DefaultHelpCommand,
    Dict,
    GroupMixin,
    HelpCommand,
    List,
    Mapping,
    Optional,
    StringView,
    T,
    Type,
    TypeVar,
    Union,
    _default,
    _DefaultRepr,
    _is_submodule,
    annotations,
    asyncio,
    collections,
    errors,
    importlib,
    inspect,
    sys,
    traceback,
    types,
    versacord,
    when_mentioned,
    when_mentioned_or,
)

__all__ = ("when_mentioned", "when_mentioned_or", "Bot", "AutoShardedBot")
