# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.types.widget import (
    Activity,
    List,
    Optional,
    Snowflake,
    TypedDict,
    User,
    Widget,
    WidgetChannel,
    WidgetMember,
    WidgetSettings,
    _WidgetOptional,
)
