# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.types.emoji import (
    EditEmoji,
    Emoji,
    Optional,
    PartialEmoji,
    Snowflake,
    SnowflakeList,
    TypedDict,
    User,
)
