# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.types.sticker import (
    BaseSticker,
    CreateGuildSticker,
    EditGuildSticker,
    GuildSticker,
    List,
    ListPremiumStickerPacks,
    Literal,
    Snowflake,
    StandardSticker,
    Sticker,
    StickerFormatType,
    StickerItem,
    StickerPack,
    TypedDict,
    Union,
    User,
    _CreateGuildStickerOptional,
    _GuildStickerOptional,
    annotations,
)
