# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.partial_emoji import (
    PE,
    TYPE_CHECKING,
    Any,
    Asset,
    AssetMixin,
    Dict,
    InvalidArgument,
    Optional,
    PartialEmoji,
    Type,
    TypeVar,
    Union,
    _EmojiTag,
    annotations,
    re,
    utils,
)

__all__ = "PartialEmoji"
