# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.types.member import (
    Member,
    MemberWithUser,
    Nickname,
    PartialMember,
    SnowflakeList,
    TypedDict,
    User,
    UserWithMember,
    _OptionalMemberWithUser,
)
