# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.utils import (
    _IS_ASCII,
    _MARKDOWN_ESCAPE_COMMON,
    _MARKDOWN_ESCAPE_REGEX,
    _MARKDOWN_ESCAPE_SUBREGEX,
    _MARKDOWN_STOCK_REGEX,
    _URL_REGEX,
    DISCORD_EPOCH,
    HAS_ORJSON,
    MISSING,
    PY_310,
    TYPE_CHECKING,
    Any,
    AsyncIterator,
    CachedSlotProperty,
    Callable,
    Dict,
    ForwardRef,
    Generic,
    InvalidArgument,
    Iterable,
    Iterator,
    List,
    Literal,
    Mapping,
    Optional,
    Protocol,
    Sequence,
    SequenceProxy,
    SnowflakeList,
    T,
    T_co,
    TimestampStyle,
    Tuple,
    Type,
    TypeVar,
    Union,
    _achunk,
    _bytes_to_base64_data,
    _cached_property,
    _chunk,
    _get_mime_type_for_image,
    _isawaitable,
    _Iter,
    _MissingSentinel,
    _signature,
    annotations,
    array,
    as_chunks,
    async_all,
    asyncio,
    attrgetter,
    b64encode,
    bisect_left,
    cached_property,
    cached_slot_property,
    classproperty,
    compute_timedelta,
    copy_doc,
    datetime,
    deprecated,
    escape_markdown,
    escape_mentions,
    evaluate_annotation,
    find,
    flatten_literal_params,
    format_dt,
    from_json,
    functools,
    get,
    get_as_snowflake,
    get_slots,
    json,
    maybe_coroutine,
    normalise_optional_params,
    oauth_url,
    overload,
    parse_ratelimit_header,
    parse_raw_channel_mentions,
    parse_raw_mentions,
    parse_raw_role_mentions,
    parse_time,
    re,
    remove_markdown,
    resolve_annotation,
    resolve_invite,
    resolve_template,
    sane_wait_for,
    sleep_until,
    snowflake_time,
    string_width,
    sys,
    time_snowflake,
    to_json,
    unicodedata,
    unique,
    utcnow,
    valid_icon_size,
    warnings,
)

__all__ = (
    "oauth_url",
    "snowflake_time",
    "time_snowflake",
    "find",
    "get",
    "sleep_until",
    "utcnow",
    "remove_markdown",
    "escape_markdown",
    "escape_mentions",
    "parse_raw_mentions",
    "parse_raw_role_mentions",
    "parse_raw_channel_mentions",
    "as_chunks",
    "format_dt",
)
