# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.components import (
    MISSING,
    TYPE_CHECKING,
    ActionRow,
    Any,
    Button,
    ButtonStyle,
    C,
    ClassVar,
    Component,
    ComponentType,
    Dict,
    List,
    Optional,
    PartialEmoji,
    SelectMenu,
    SelectOption,
    Tuple,
    Type,
    TypeVar,
    Union,
    _component_factory,
    _EmojiTag,
    annotations,
    get_slots,
    try_enum,
)

__all__ = ("Component", "ActionRow", "Button", "SelectMenu", "SelectOption")
