# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.ext.commands.__init__ import (
    ArgumentParsingError,
    AutoShardedBot,
    BadArgument,
    BadBoolArgument,
    BadColorArgument,
    BadColourArgument,
    BadFlagArgument,
    BadInviteArgument,
    BadLiteralArgument,
    BadUnionArgument,
    Bot,
    BotMissingAnyRole,
    BotMissingPermissions,
    BotMissingRole,
    BucketType,
    CategoryChannelConverter,
    ChannelNotFound,
    ChannelNotReadable,
    CheckAnyFailure,
    CheckFailure,
    Cog,
    CogMeta,
    ColorConverter,
    ColourConverter,
    Command,
    CommandError,
    CommandInvokeError,
    CommandNotFound,
    CommandOnCooldown,
    CommandRegistrationError,
    Context,
    ConversionError,
    Converter,
    Cooldown,
    CooldownMapping,
    DefaultHelpCommand,
    DisabledCommand,
    DynamicCooldownMapping,
    EmojiConverter,
    EmojiNotFound,
    ExpectedClosingQuoteError,
    ExtensionAlreadyLoaded,
    ExtensionError,
    ExtensionFailed,
    ExtensionNotFound,
    ExtensionNotLoaded,
    Flag,
    FlagConverter,
    FlagError,
    GameConverter,
    Greedy,
    Group,
    GroupMixin,
    GuildChannelConverter,
    GuildConverter,
    GuildNotFound,
    GuildStickerConverter,
    GuildStickerNotFound,
    HelpCommand,
    IDConverter,
    InvalidEndOfQuotedStringError,
    InviteConverter,
    MaxConcurrency,
    MaxConcurrencyReached,
    MemberConverter,
    MemberNotFound,
    MessageConverter,
    MessageNotFound,
    MinimalHelpCommand,
    MissingAnyRole,
    MissingFlagArgument,
    MissingPermissions,
    MissingRequiredArgument,
    MissingRequiredFlag,
    MissingRole,
    NoEntryPointError,
    NoPrivateMessage,
    NotOwner,
    NSFWChannelRequired,
    ObjectConverter,
    ObjectNotFound,
    Paginator,
    PartialEmojiConversionFailure,
    PartialEmojiConverter,
    PartialMessageConverter,
    PrivateMessageOnly,
    RoleConverter,
    RoleNotFound,
    StageChannelConverter,
    TextChannelConverter,
    ThreadConverter,
    ThreadNotFound,
    TooManyArguments,
    TooManyFlags,
    UnexpectedQuoteError,
    UserConverter,
    UserInputError,
    UserNotFound,
    VoiceChannelConverter,
    after_invoke,
    before_invoke,
    bot_has_any_role,
    bot_has_guild_permissions,
    bot_has_permissions,
    bot_has_role,
    check,
    check_any,
    clean_content,
    command,
    cooldown,
    dm_only,
    dynamic_cooldown,
    flag,
    group,
    guild_only,
    has_any_role,
    has_guild_permissions,
    has_permissions,
    has_role,
    is_nsfw,
    is_owner,
    max_concurrency,
    run_converters,
    when_mentioned,
    when_mentioned_or,
)
