# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.context_managers import (
    TYPE_CHECKING,
    Optional,
    Type,
    TypeVar,
    Typing,
    _typing_done_callback,
    annotations,
    asyncio,
)

__all__ = "Typing"
