# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.types.guild import (
    Ban,
    ChannelPositionUpdate,
    DefaultMessageNotificationLevel,
    Emoji,
    ExplicitContentFilterLevel,
    Guild,
    GuildChannel,
    GuildFeature,
    GuildPreview,
    GuildPrune,
    GuildVoiceState,
    GuildWithCounts,
    InviteGuild,
    List,
    Literal,
    Member,
    MFALevel,
    NSFWLevel,
    Optional,
    PartialPresenceUpdate,
    PremiumTier,
    Role,
    RolePositionUpdate,
    Snowflake,
    Thread,
    TypedDict,
    UnavailableGuild,
    User,
    VerificationLevel,
    WelcomeScreen,
    _BaseGuildPreview,
    _GuildOptional,
    _GuildPreviewUnique,
    _RolePositionRequired,
    _UnavailableGuildOptional,
)
