# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.types.activity import (
    Activity,
    ActivityAssets,
    ActivityButton,
    ActivityEmoji,
    ActivityParty,
    ActivitySecrets,
    ActivityTimestamps,
    ActivityType,
    ClientStatus,
    List,
    Literal,
    Optional,
    PartialPresenceUpdate,
    PartialUser,
    SendableActivity,
    Snowflake,
    StatusType,
    TypedDict,
    _ActivityEmojiOptional,
    _BaseActivity,
    _SendableActivityOptional,
    annotations,
)
