# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.webhook.async_ import (
    MISSING,
    TYPE_CHECKING,
    Any,
    Asset,
    AsyncDeferredLock,
    AsyncWebhookAdapter,
    BaseUser,
    BaseWebhook,
    ContextVar,
    Dict,
    DiscordServerError,
    ExecuteWebhookParameters,
    Forbidden,
    Hashable,
    HTTPException,
    InvalidArgument,
    List,
    Literal,
    Message,
    NamedTuple,
    NotFound,
    Optional,
    PartialMessageable,
    PartialWebhookChannel,
    PartialWebhookGuild,
    Route,
    Tuple,
    Union,
    User,
    Webhook,
    WebhookMessage,
    WebhookType,
    _FriendlyHttpAttributeErrorHelper,
    _log,
    _WebhookState,
    aiohttp,
    annotations,
    async_context,
    asyncio,
    handle_message_parameters,
    json,
    logging,
    overload,
    re,
    try_enum,
    urlquote,
    utils,
)

__all__ = ("Webhook", "WebhookMessage", "PartialWebhookChannel", "PartialWebhookGuild")
