# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.ext.tasks.__init__ import (
    ET,
    FT,
    LF,
    MISSING,
    Any,
    Awaitable,
    Callable,
    ExponentialBackoff,
    Generic,
    List,
    Loop,
    Optional,
    Sequence,
    SleepHandle,
    T,
    Type,
    TypeVar,
    Union,
    _func,
    aiohttp,
    annotations,
    asyncio,
    datetime,
    inspect,
    loop,
    sys,
    traceback,
    versacord,
)

__all__ = "loop"
