# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.threads import (
    MISSING,
    TYPE_CHECKING,
    Callable,
    ChannelType,
    ClientException,
    Dict,
    Hashable,
    Iterable,
    List,
    Messageable,
    Optional,
    Thread,
    ThreadMember,
    Union,
    annotations,
    asyncio,
    get_as_snowflake,
    parse_time,
    time,
    try_enum,
)

__all__ = ("Thread", "ThreadMember")
