# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.ext.commands.cog import (
    MISSING,
    TYPE_CHECKING,
    Any,
    Callable,
    ClassVar,
    Cog,
    CogMeta,
    CogT,
    Dict,
    FuncT,
    Generator,
    List,
    Optional,
    Tuple,
    Type,
    TypeVar,
    _BaseCommand,
    _cog_special_method,
    annotations,
    inspect,
    versacord,
)

__all__ = ("CogMeta", "Cog")
