# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.appinfo import (
    TYPE_CHECKING,
    AppInfo,
    Asset,
    List,
    Optional,
    PartialAppInfo,
    annotations,
    utils,
)

__all__ = ("AppInfo", "PartialAppInfo")
