# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.types.audit_log import (
    AuditEntryInfo,
    AuditLog,
    AuditLogChange,
    AuditLogEntry,
    AuditLogEvent,
    ChannelType,
    DefaultMessageNotificationLevel,
    ExplicitContentFilterLevel,
    IntegrationExpireBehavior,
    List,
    Literal,
    MFALevel,
    Optional,
    PartialIntegration,
    PermissionOverwrite,
    Role,
    Snowflake,
    Thread,
    TypedDict,
    Union,
    User,
    VerificationLevel,
    VideoQualityMode,
    Webhook,
    _AuditLogChange_AssetHash,
    _AuditLogChange_Bool,
    _AuditLogChange_ChannelType,
    _AuditLogChange_DefaultMessageNotificationLevel,
    _AuditLogChange_ExplicitContentFilter,
    _AuditLogChange_Int,
    _AuditLogChange_IntegrationExpireBehaviour,
    _AuditLogChange_ListRole,
    _AuditLogChange_MFALevel,
    _AuditLogChange_Overwrites,
    _AuditLogChange_Snowflake,
    _AuditLogChange_Str,
    _AuditLogChange_VerificationLevel,
    _AuditLogChange_VideoQualityMode,
    _AuditLogEntryOptional,
    annotations,
)
