# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.object import TYPE_CHECKING, Hashable, Object, SupportsInt, Union, annotations, utils

__all__ = "Object"
