# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.mixins import EqualityComparable, Hashable

__all__ = ("EqualityComparable", "Hashable")
