# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.channel import (
    DMC,
    MISSING,
    TYPE_CHECKING,
    Any,
    ArchivedThreadIterator,
    Asset,
    Callable,
    CategoryChannel,
    ChannelType,
    ClientException,
    Dict,
    DMChannel,
    GroupChannel,
    Hashable,
    InvalidArgument,
    Iterable,
    List,
    Mapping,
    Object,
    Optional,
    PartialMessageable,
    PermissionOverwrite,
    Permissions,
    StageChannel,
    StageInstance,
    StagePrivacyLevel,
    TextChannel,
    Thread,
    Tuple,
    Type,
    TypeVar,
    Union,
    VideoQualityMode,
    VocalGuildChannel,
    VoiceChannel,
    VoiceRegion,
    _channel_factory,
    _guild_channel_factory,
    _single_delete_strategy,
    _threaded_channel_factory,
    _threaded_guild_channel_factory,
    abc,
    annotations,
    asyncio,
    datetime,
    overload,
    time,
    try_enum,
    utils,
)

__all__ = (
    "TextChannel",
    "VoiceChannel",
    "StageChannel",
    "DMChannel",
    "CategoryChannel",
    "GroupChannel",
    "PartialMessageable",
)
