# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.ui.view import (
    TYPE_CHECKING,
    ActionRowComponent,
    Any,
    ButtonComponent,
    Callable,
    ClassVar,
    Component,
    Dict,
    Item,
    ItemCallbackType,
    Iterator,
    List,
    Optional,
    SelectComponent,
    Sequence,
    Tuple,
    View,
    ViewStore,
    _component_factory,
    _component_to_item,
    _ViewWeights,
    _walk_all_components,
    annotations,
    asyncio,
    groupby,
    os,
    partial,
    sys,
    time,
    traceback,
)

__all__ = "View"
