# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.types.voice import (
    GuildVoiceState,
    List,
    Literal,
    MemberWithUser,
    Optional,
    Snowflake,
    SupportedModes,
    TypedDict,
    VoiceIdentify,
    VoiceReady,
    VoiceRegion,
    VoiceServerUpdate,
    VoiceState,
    _PartialVoiceStateOptional,
    _VoiceState,
)
