# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.emoji import (
    MISSING,
    TYPE_CHECKING,
    Any,
    Asset,
    AssetMixin,
    Emoji,
    Iterator,
    List,
    Optional,
    PartialEmoji,
    SnowflakeList,
    Tuple,
    User,
    _EmojiTag,
    annotations,
    snowflake_time,
)

__all__ = "Emoji"
