# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.iterators import (
    OLDEST_OBJECT,
    OT,
    TYPE_CHECKING,
    Any,
    ArchivedThreadIterator,
    AsyncIterator,
    AuditLogEntry,
    AuditLogIterator,
    Awaitable,
    Callable,
    GuildIterator,
    HistoryIterator,
    List,
    MemberIterator,
    NoMoreItems,
    Object,
    Optional,
    ReactionIterator,
    T,
    TypeVar,
    Union,
    _AsyncIterator,
    _ChunkedAsyncIterator,
    _FilteredAsyncIterator,
    _Func,
    _identity,
    _MappedAsyncIterator,
    annotations,
    asyncio,
    datetime,
    maybe_coroutine,
    snowflake_time,
    time_snowflake,
)

__all__ = (
    "ReactionIterator",
    "HistoryIterator",
    "AuditLogIterator",
    "GuildIterator",
    "MemberIterator",
)
