# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.ext.commands.help import (
    TYPE_CHECKING,
    Command,
    CommandError,
    DefaultHelpCommand,
    Group,
    HelpCommand,
    MinimalHelpCommand,
    Optional,
    Paginator,
    _HelpCommandImpl,
    _not_overriden,
    copy,
    functools,
    inspect,
    itertools,
    re,
    versacord,
)

__all__ = ("Paginator", "HelpCommand", "DefaultHelpCommand", "MinimalHelpCommand")
