# SPDX-License-Identifier: MIT
"""
Aliased moodule. See the same file in the versacord folder for more information
Autogenerated by aliasgen.py
"""

from versacord.client import (
    MISSING,
    TYPE_CHECKING,
    ActivityTypes,
    AllowedMentions,
    Any,
    AppInfo,
    ApplicationFlags,
    BaseActivity,
    Callable,
    ChannelType,
    Client,
    ClientException,
    ClientUser,
    ConnectionClosed,
    ConnectionState,
    Coro,
    Coroutine,
    Dict,
    DiscordException,
    DiscordServerError,
    DiscordVoiceWebSocket,
    DiscordWebSocket,
    Emoji,
    ExponentialBackoff,
    Forbidden,
    GatewayNotFound,
    Generator,
    Guild,
    GuildIterator,
    GuildSticker,
    HTTPClient,
    HTTPException,
    Intents,
    InteractionResponded,
    InvalidArgument,
    InvalidData,
    Invite,
    KeepAliveHandler,
    List,
    LoginFailure,
    NoMoreItems,
    NotFound,
    Object,
    Optional,
    PartialMessageable,
    PrivilegedIntentsRequired,
    ReconnectWebSocket,
    Sequence,
    StageInstance,
    StandardSticker,
    Status,
    StickerPack,
    Template,
    Thread,
    Tuple,
    TypeVar,
    Union,
    User,
    View,
    VoiceClient,
    VoiceKeepAliveHandler,
    VoiceRegion,
    Webhook,
    Widget,
    _cancel_tasks,
    _cleanup_loop,
    _log,
    _sticker_factory,
    _threaded_channel_factory,
    aiohttp,
    annotations,
    asyncio,
    create_activity,
    logging,
    signal,
    sys,
    traceback,
    utils,
)

__all__ = "Client"
